#ifndef CMPLR_LITERAL_HPP_INCLUDED
#define CMPLR_LITERAL_HPP_INCLUDED


#include"typesystem_element.hpp"
#include"cmplr_constant.hpp"
#include"mkf_node.hpp"
#include<vector>




struct Declaration;
struct Function;
struct PreContext;


namespace expression{
struct Node;
}


struct Literal;
struct Subscript;


struct         Array: public std::vector<Literal>{};
struct  ArgumentList: public Array               {};


enum class
LiteralKind
{
  null,

  expression,

  array,
  literal,

  null_pointer,
  boolean,
  string,
  character,
  integer,
  identifier,
  argument_list,
  subscript,

  reference,
  function,

};


struct
Literal
{
  LiteralKind  kind;

  union{
    int  i;

    std::string*    id;
    std::u16string*  s;

    Array*         arr;
    ArgumentList*  args;
    Subscript*     subsc;

    expression::Node*  nd;

    Literal*  lit;

    Function*  fn;

    const Declaration*  decl;

  } data;


  Literal();
  Literal( int32_t  i);
  Literal(char16_t  c);
  Literal(nullptr_t  nul);
  Literal(bool  b);
  Literal(std::u16string*  s);
  Literal(std::string*  id);
  Literal(expression::Node*  nd);
  Literal(Literal*  lit);
  Literal(Array*  arr);
  Literal(Function*  fn);
  Literal(ArgumentList*  args);
  Literal(Subscript*    subsc);
  Literal(const mkf::Node&  src, PreContext&  prectx);
  Literal(const Literal&   rhs) noexcept;
  Literal(      Literal&&  rhs) noexcept;
 ~Literal();

  Literal&  operator=(const Literal&   rhs) noexcept;
  Literal&  operator=(      Literal&&  rhs) noexcept;

  void  clear();


  void  print(FILE*  f=stdout) const;

  Constant  make_constant(PreContext&  prectx) const;

  Type  compile_value(Context&  ctx) const;

  void  compile_data(const Type&  type, Context&  ctx) const;

  void  read(const mkf::Node&  src, PreContext&  prectx);

  static Array  read_list(const mkf::Node&  src, PreContext&  prectx);

  static uint32_t          read_integer_literal(const mkf::Node&  src);
  static char16_t        read_character_literal(const mkf::Node&  src);
  static std::u16string     read_string_literal(const mkf::Node&  src);

};




struct  Subscript: public Literal{};
struct      Print: public Literal{};
struct     Return: public Literal{};




#endif




