#include"cmplr_statement.hpp"
#include"cmplr_block.hpp"
#include"cmplr_function.hpp"
#include"expression_node.hpp"
#include"cmplr_declaration.hpp"
#include"cmplr_precontext.hpp"




void
Statement::
read(const mkf::Node&  src, PreContext&  prectx)
{
  mkf::Cursor  cur(src);

    while(!cur.test_ended())
    {
      auto&  nd = cur.get();

        if(nd == "declaration")
        {
          *this = Statement(prectx.append_declaration(Declaration(nd,prectx)));
        }

      else
        if(nd == "control_statement")
        {
          read_control_statement(nd,prectx);
        }

      else
        if(nd == "print_statement")
        {
          read_print_statement(nd,prectx);
        }

      else
        if(nd == "halt")
        {
          *this = Statement(Halt());
        }

      else
        if(nd == "block")
        {
          auto&  blk = prectx.function->make_block(BlockKind::plain,0,nd,prectx);

          *this = Statement(&blk);
        }

      else
        if(nd == "expression")
        {
          *this = Statement(new expression::Node(nd,prectx));
        }


      cur.advance();
    }
}


void
Statement::
read_print_statement(const mkf::Node&  src, PreContext&  prectx)
{
  Print*  prn = nullptr;

  mkf::Cursor  cur(src);

    while(!cur.test_ended())
    {
      auto&  nd = cur.get();

        if(nd == "expression")
        {
          prn = static_cast<Print*>(new Literal(new expression::Node(nd,prectx)));
        }

      else
        if(nd == "literal_object")
        {
          prn = static_cast<Print*>(new Literal(nd,prectx));
        }


      cur.advance();
    }


  *this = Statement(prn);
}




