#include"typesystem_element.hpp"





namespace typesystem{


bool  Element::is_void()      const{return(is_fundamental() && (data.fdm.kind == FundamentalKind::void_));}
bool  Element::is_valid()      const{return !is_void();}
bool  Element::is_function()  const{return(kind == TypeKind::function);}
bool  Element::is_nullptr()   const{return(is_fundamental() && (data.fdm.kind == FundamentalKind::nullptr_));}
bool  Element::is_boolean()   const{return(is_fundamental() && (data.fdm.kind == FundamentalKind::bool_));}
bool  Element::is_pointer()   const{return(kind == TypeKind::pointer);}
bool  Element::is_array()     const{return(kind == TypeKind::array);}
bool  Element::is_reference() const{return(kind == TypeKind::reference);}
bool  Element::is_enum()      const{return(kind == TypeKind::enum_);}
bool  Element::is_struct()    const{return(kind == TypeKind::struct_);}
bool  Element::is_union()     const{return(kind == TypeKind::union_);}


bool
Element::
is_integer() const
{
  return(is_fundamental() &&
         ((data.fdm.kind == FundamentalKind::char_ ) ||
          (data.fdm.kind == FundamentalKind::bool_ ) ||
          (data.fdm.kind == FundamentalKind::int8  ) ||
          (data.fdm.kind == FundamentalKind::uint8 ) ||
          (data.fdm.kind == FundamentalKind::int16 ) ||
          (data.fdm.kind == FundamentalKind::uint16) ||
          (data.fdm.kind == FundamentalKind::int32 )));
}


bool  Element::is_arithmetic() const{return is_integer();}
bool  Element::is_fundamental() const{return (kind == TypeKind::fundamental);}
bool  Element::is_object() const{return(is_enum()||is_struct()||is_union()||is_array()||is_scalar());}
bool  Element::is_scalar() const{return(is_arithmetic()||is_enum()||is_pointer()||is_nullptr());}
bool  Element::is_compound() const{return(is_array()||is_function()||is_pointer()||is_reference()||is_struct()||is_union()||is_enum());}


bool
Element::
is_literal() const
{
  return(is_scalar()||
           is_void()||
         (is_reference() && data.ptr.referred->is_literal())|
         (is_array()     && data.arr.element->is_literal()));
}


bool
Element::
is_signed() const
{
  return(is_fundamental() &&
         ((data.fdm.kind == FundamentalKind::int8  ) ||
          (data.fdm.kind == FundamentalKind::int16 ) ||
          (data.fdm.kind == FundamentalKind::int32 )));
}


bool
Element::
is_unsigned() const
{
  return(is_fundamental() &&
         ((data.fdm.kind == FundamentalKind::char_ ) ||
          (data.fdm.kind == FundamentalKind::bool_ ) ||
          (data.fdm.kind == FundamentalKind::uint8 ) ||
          (data.fdm.kind == FundamentalKind::uint16)));
}




bool
Element::
is_able_to_convert_implicitly(const Element&  target) const
{
    if(*this == target)
    {
      return true;
    }


    switch(kind)
    {
  case(ElementKind::fundamental):
      return data.fdm.is_able_to_convert_implicitly(target);
      break;
  case(ElementKind::array):
      return(target == data.arr.element->make_pointer());
      break;
  case(ElementKind::pointer):
      return target.is_boolean();
      break;
  case(ElementKind::reference):
      return(*data.ptr.referred == target);
      break;
  case(ElementKind::struct_):
      break;
  case(ElementKind::union_):
      break;
  case(ElementKind::enum_):
      return(target.is_integer() ||
             target.is_boolean());
      break;
  case(ElementKind::function):
      break;
  case(ElementKind::argument_list):
      break;
    }


  return false;
}




bool
Element::
is_able_to_reinterpret(const Element&  target) const
{
    if(*this == target)
    {
      return true;
    }


    switch(kind)
    {
  case(ElementKind::fundamental):
      return data.fdm.is_able_to_reinterpret(target);
      break;
  case(ElementKind::array):
      break;
  case(ElementKind::pointer):
      return(target.is_pointer() ||
             target.is_boolean() ||
             target.is_pointer);
      break;
  case(ElementKind::reference):
      return target.is_reference();
      break;
  case(ElementKind::struct_):
      break;
  case(ElementKind::union_):
      break;
  case(ElementKind::enum_):
      return(target.is_integer() ||
             target.is_boolean());
      break;
  case(ElementKind::function):
      break;
  case(ElementKind::argument_list):
      break;
    }


  return false;
}




}




